﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis {
  [StorableClass]
  [Item("MedianThresholdClassificationCalculator", "")]
  public class MedianThresholdClassificationCalculator : MedianThresholdCalculator {

    public MedianThresholdClassificationCalculator()
      : base() {
    }
    [StorableConstructor]
    protected MedianThresholdClassificationCalculator(bool deserializing) : base(deserializing) { }
    protected MedianThresholdClassificationCalculator(MedianThresholdClassificationCalculator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MedianThresholdClassificationCalculator(this, cloner);
    }

    protected override double DiscriminantAggregateEstimatedClassValues(IDictionary<IClassificationSolution, double> estimatedClassValues, IDictionary<IDiscriminantFunctionClassificationSolution, double> estimatedValues) {
      IList<double> values = estimatedValues.Select(x => x.Value).ToList();
      if (values.Count <= 0)
        return double.NaN;
      double median = GetMedian(values);
      return GetClassValueToMedian(median);
    }
    private double GetClassValueToMedian(double median) {
      double classValue = classValues.First();
      for (int i = 0; i < classValues.Count(); i++) {
        if (median > threshold[i])
          classValue = classValues[i];
        else
          break;
      }
      return classValue;
    }
  }
}
